/**
 * 
 */
package com.opswat.metadefender.oauth.client.controller;

import java.io.IOException;
import java.util.ArrayList;

import javax.servlet.http.HttpSession;

import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.map.DeserializationConfig;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.map.annotate.JsonSerialize.Inclusion;
import org.codehaus.jackson.node.ArrayNode;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;

import com.opswat.metadefender.oauth.client.domain.Account;
import com.opswat.metadefender.oauth.client.domain.Endpoint;
import com.opswat.metadefender.oauth.client.domain.Eventlog;
import com.opswat.metadefender.oauth.client.utils.Common;
import com.opswat.metadefender.oauth.client.utils.Define;

/**
 * @author Cao Hong Phuoc
 *
 */

@Controller
public class PageController {
    
    @RequestMapping(value = "/account")
    public String getAccountInfo(HttpSession session, Model model) throws Exception {
        String accessToken = Common.getAccessToken(session);
        if (accessToken == null) {
            return "redirect:/authorize";
        }

        String data = Common.getData(Define.API_ACCOUNT, accessToken);
        Account account = convertToObject(data, Account.class);
        
        model.addAttribute("account", account);
        return "account";
    }
    
    @RequestMapping(value = "/accountconfig")
    public String getAccountConfig(HttpSession session, Model model) {
        
        String accessToken = Common.getAccessToken(session);
        if (accessToken == null) {
            return "redirect:/authorize";
        }
        
        String data = Common.getData(Define.API_ACCOUNT_CONFIG, accessToken);
        model.addAttribute("config", data);
        return "accountconfig";
    }
    
    @RequestMapping(value = "/devices")
    public String getDevices(HttpSession session, Model model) throws Exception {
        
        String accessToken = Common.getAccessToken(session);
        if (accessToken == null) {
            return "redirect:/authorize";
        }
        
        ArrayList<Endpoint> devices = new ArrayList<Endpoint>();
        ObjectMapper objectMapper = new ObjectMapper();
        String uri = Define.API_DEVICES + "&access_token=" + accessToken;
        String data = Common.getData(uri);
        ArrayNode arrNode = (ArrayNode) objectMapper.readTree(data);
        
        for (JsonNode dataNode:arrNode) {
        	Endpoint endpoint = convertToObject(dataNode, Endpoint.class);
            devices.add(endpoint);
        }
        
        model.addAttribute("devices", devices);
        return "devices";
    }
    
    @RequestMapping(value = "/devices/{id}")
    public String getDeviceDetail(HttpSession session, Model model, @PathVariable("id") String endpointId) throws Exception {
        
        String accessToken = Common.getAccessToken(session);
        if (accessToken == null) {
            return "redirect:/authorize";
        }
        
        String uri = Define.API_DEVICES_DETAIL + "/" + endpointId;
        String data = Common.getData(uri, accessToken);
        Endpoint endpoint = convertToObject(data, Endpoint.class);
        
        model.addAttribute("endpoint", endpoint);
        return "device";
    }
    
    @RequestMapping(value = "/logs")
    public String getLogs(HttpSession session, Model model) throws Exception {
        
        String accessToken = Common.getAccessToken(session);
        if (accessToken == null) {
            return "redirect:/authorize";
        }
        
        ObjectMapper objectMapper = new ObjectMapper();
        ArrayList<Eventlog> logs = new ArrayList<Eventlog>();
        
        String result = Common.getData(Define.API_LOGS, accessToken);
        JsonNode readTree = objectMapper.readTree(result).get("logs");
        ArrayNode data = (ArrayNode) readTree;
        
        for (JsonNode dataNode:data) {
        	Eventlog log = convertToObject(dataNode, Eventlog.class);
            logs.add(log);
        }
        
        model.addAttribute("logs", logs);
        return "logs";
    }
    
    public <T> T convertToObject(String content, Class<T> clazz) throws IOException {
    	ObjectMapper objectMapper = new ObjectMapper();
    	objectMapper.disable(DeserializationConfig.Feature.FAIL_ON_UNKNOWN_PROPERTIES);
    	objectMapper.configure(DeserializationConfig.Feature.FAIL_ON_UNKNOWN_PROPERTIES, false);
    	objectMapper.setSerializationInclusion(Inclusion.NON_NULL);
        return objectMapper.readValue(content, clazz);
    }
    
    public <T> T convertToObject(JsonNode dataNode, Class<T> clazz) throws IOException {
    	ObjectMapper objectMapper = new ObjectMapper();
    	objectMapper.disable(DeserializationConfig.Feature.FAIL_ON_UNKNOWN_PROPERTIES);
    	objectMapper.configure(DeserializationConfig.Feature.FAIL_ON_UNKNOWN_PROPERTIES, false);
    	objectMapper.setSerializationInclusion(Inclusion.NON_NULL);
        return objectMapper.readValue(dataNode, clazz);
    }
    
    @RequestMapping(value = "/logout")
    public String logout(HttpSession session) {
        session.invalidate();
        return "redirect:/";
    }
}
